@@ -4,7 +4,7 @@ from struct import Struct, unpack
 from sqlite3 import connect
 
 
-def stream_sqlite(sqlite_chunks, chunk_size=65536):
+def stream_sqlite(sqlite_chunks):
     INTERIOR_INDEX = b'\x02'
     INTERIOR_TABLE = b'\x05'
     LEAF_INDEX = b'\x0a'
@@ -30,13 +30,11 @@ def stream_sqlite(sqlite_chunks, chunk_size=65536):
                         chunk = next(it)
                     except StopIteration:
                         raise ValueError('Fewer bytes than expected in SQLite stream') from None
-                prev_offset = offset
-                prev_chunk = chunk
-                to_yield = min(num, len(chunk) - offset, chunk_size)
+                to_yield = min(num, len(chunk) - offset)
+                yield chunk[offset:offset + to_yield]
+                num -= to_yield
                 offset = (offset + to_yield) % len(chunk)
                 chunk = chunk if offset else b''
-                num -= to_yield
-                yield prev_chunk[prev_offset:prev_offset + to_yield]
 
         def _get_num(num):
             return b''.join(chunk for chunk in _yield_num(num))
@@ -165,23 +163,15 @@ def stream_sqlite(sqlite_chunks, chunk_size=65536):
                     cur = con.cursor()
 
                     return tuple(
-                        {
-                            'name': cell[1],
-                            'info': schema(cur, cell[1], cell[4]),
-                            'root_page': cell[3],
-                        }
+                        (
+                            cell[1],                        # table name
+                            schema(cur, cell[1], cell[4]),  # table info
+                            cell[3],                        # root page
+                        )
                         for cell in master_cells
                         if cell[0] == 'table'
                     )
 
-            def process_if_buffered_or_remember(table_name, page_num):
-                try:
-                    page_bytes, page_reader = page_buffer.pop(page_num)
-                except KeyError:
-                    page_types[page_num] = partial(process_table_page, table_name)
-                else:
-                    yield from process_table_page(table_name, page_bytes, page_reader)
-
             page_type = page_reader(1)
             first_free_block, num_cells, cell_content_start, num_frag_free = \
                 table_header.unpack(page_reader(7))
@@ -193,9 +183,9 @@ def stream_sqlite(sqlite_chunks, chunk_size=65536):
             pointers = unpack('>{}H'.format(num_cells), page_reader(num_cells * 2))
 
             if page_type == LEAF_TABLE and table_name == 'sqlite_schema':
-                for row in get_master_table(yield_leaf_table_cells(page_bytes, pointers)):
-                    master_table[row['name']] = row['info']
-                    yield from process_if_buffered_or_remember(row['name'], row['root_page'])
+                for table_name, table_info, root_page in get_master_table(yield_leaf_table_cells(page_bytes, pointers)):
+                    master_table[table_name] = table_info
+                    yield from process_if_buffered_or_remember(partial(process_table_page, table_name), root_page)
 
             elif page_type == LEAF_TABLE:
                 table_info = master_table[table_name]
@@ -209,39 +199,35 @@ def stream_sqlite(sqlite_chunks, chunk_size=65536):
 
             elif page_type == INTERIOR_TABLE:
                 for page_num in yield_interior_table_cells(page_bytes, pointers):
-                    yield from process_if_buffered_or_remember(table_name, page_num)
-                yield from process_if_buffered_or_remember(table_name, right_most_pointer)
+                    yield from process_if_buffered_or_remember(
+                        partial(process_table_page, table_name), page_num)
+                yield from process_if_buffered_or_remember(
+                    partial(process_table_page, table_name), right_most_pointer)
 
             else:
-                raise ValueError('Unhandled page type in SQLite stream')
+                raise ValueError('Unhandled table page type in SQLite stream')
 
         def process_freelist_trunk_page(page_bytes, page_reader):
-            def trunk_process_if_buffered_or_remember(page_num):
-                try:
-                    page_bytes, page_reader = page_buffer.pop(page_num)
-                except KeyError:
-                    page_types[page_num] = process_freelist_trunk_page
-                else:
-                    yield from process_freelist_trunk_page(page_bytes, page_reader)
-
-            def leaf_process_if_buffered_or_remember(page_num):
-                try:
-                    del page_buffer[page_num]
-                except KeyError:
-                    page_types[page_num] = process_freelist_leaf_page
-
             next_trunk, num_leaves = freelist_trunk_header.unpack(page_reader(8))
             leaf_pages = unpack('>{}L'.format(num_leaves), page_reader(num_leaves * 4))
 
             for page_num in leaf_pages:
-                leaf_process_if_buffered_or_remember(page_num)
+                yield from process_if_buffered_or_remember(process_freelist_leaf_page, page_num)
 
             if next_trunk != 0:
-                yield from trunk_process_if_buffered_or_remember(next_trunk)
+                yield from process_if_buffered_or_remember(process_freelist_trunk_page, next_trunk)
 
         def process_freelist_leaf_page(page_bytes, page_reader):
             yield from []
 
+        def process_if_buffered_or_remember(process, page_num):
+            try:
+                page_bytes, page_reader = page_buffer.pop(page_num)
+            except KeyError:
+                page_types[page_num] = process
+            else:
+                yield from process(page_bytes, page_reader)
+
         page_types[1] = partial(process_table_page, 'sqlite_schema')
         page_types[first_freelist_trunk_page] = process_freelist_trunk_page
 
